==> 2.0.1.zip: phpcollections/src/Collections/ImmutableCollection.php <==
<?php

namespace jrosset\Collections;

use ArrayIterator;
use Closure;
use InvalidArgumentException;
use JsonException;
use Traversable;

/**
 * An immutable collection
 *
 * @psalm-template     TKey of array-key
 * @template-covariant TValue
 * @template-implements IImmutableCollection<TKey, TValue>
 */
class ImmutableCollection implements IImmutableCollection {
    /**
     * @var array The internal array of elements
     * @psalm-var array<TKey, TValue>
     *
     * @internal
     */
    protected array $elements;

    /**
     * Initialise a new collection
     *
     * @param array|Traversable|null $other The initial values
     *
     * @psalm-param array<TKey, TValue>|Traversable<TKey, TValue>|null
     *
     * @throws InvalidArgumentException If the initial values aren't valid
     */
    public function __construct ($other = null) {
        $this->_initialize($other);
    }

    /**
     * Initialise the internals elements
     *
     * @param array|Traversable|null $other The initial values
     *
     * @psalm-param array<TKey, TValue>|Traversable<TKey, TValue>|null
     *
     * @throws InvalidArgumentException If the initial values aren't valid
     * @internal
     *
     */
    protected function _initialize ($other = null) {
        $this->elements = [];
        if ($other !== null) {
            if (!is_array($other) && !$other instanceof Traversable) {
                throw new InvalidArgumentException('The initial values must be an array or a ' . IArrayCast::class . ' instance');
            }

            foreach ($other as $key => $value) {
                $this->_set($key, $value);
            }
        }
    }
    /**
     * Internally set a value to a key
     *
     * @param array-key    $key   The key
     * @param mixed        $value The value
     *
     * @psalm-param TKey   $key
     * @psalm-param TValue $value
     *
     * @return $this
     * @internal
     *
     */
    protected function _set ($key, $value): self {
        $this->elements[$this->_normalizeKey($key)] = $value;
        return $this;
    }
    /**
     * Internally add a value
     *
     * @param mixed        $value The value
     *
     * @psalm-param TValue $value
     *
     * @return $this
     * @internal
     *
     */
    protected function _add ($value): self {
        $this->elements[] = $value;
        return $this;
    }
    /**
     * Normalize a key
     *
     * @param array-key  $key The key to normalize
     *
     * @psalm-param TKey $key
     *
     * @return array-key The normalized key
     * @psalm-return TKey
     */
    protected function _normalizeKey ($key) {
        return $key;
    }

    /**
     * @inheritDoc
     */
    public function getIterator () {
        return new ArrayIterator($this->elements);
    }

    /**
     * @inheritDoc
     */
    public function offsetExists ($offset): bool {
        return $this->exists($offset);
    }
    /**
     * @inheritDoc
     */
    public function offsetGet ($offset) {
        return $this->get($offset);
    }
    /**
     * @inheritDoc
     */
    public function offsetSet ($offset, $value): void {
        throw new ImmutableException();
    }
    /**
     * @inheritDoc
     */
    public function offsetUnset ($offset): void {
        throw new ImmutableException();
    }

    /**
     * @inheritDoc
     */
    public function serialize (): ?string {
        return serialize($this->elements);
    }
    /**
     * @inheritDoc
     */
    public function unserialize ($data) {
        $this->_initialize(unserialize($data));
    }

    /**
     * @inheritDoc
     */
    public function toArray (): array {
        return $this->elements;
    }

    /**
     * @inheritDoc
     */
    public function count (): int {
        return count($this->elements);
    }
    /**
     * @inheritDoc
     */
    public function empty (): bool {
        return $this->count() === 0;
    }

    /**
     * @inheritDoc
     */
    public function exists ($key): bool {
        return isset($this->elements[$this->_normalizeKey($key)]);
    }
    /**
     * @inheritDoc
     */
    public function contains ($value, bool $strict = false): bool {
        return in_array($value, $this->elements, $strict);
    }

    /**
     * @inheritDoc
     */
    public function get ($key) {
        return $this->elements[$this->_normalizeKey($key)] ?? null;
    }
    /**
     * @inheritDoc
     */
    public function key ($value, bool $strict = false) {
        foreach ($this->elements as $currentKey => $currentValue) {
            if (($strict && $value === $currentValue) || (!$strict && $value == $currentValue)) {
                return $currentKey;
            }
        }
        return null;
    }

    /**
     * @inheritDoc
     */
    public function slice (int $offset, ?int $length = null): IImmutableCollection {
        $output = new static();

        $currentIndex = 0;
        foreach ($this->elements as $key => $value) {
            if ($currentIndex++ < $offset) {
                continue;
            }

            $output->_set($key, $value);
            if ($output->count() >= $length) {
                break;
            }
        }

        return $output;
    }

    /**
     * @inheritDoc
     */
    public function filter (Closure $filter): IImmutableCollection {
        $output = new static();
        foreach ($this->elements as $key => $value) {
            if (!$filter($key, $value)) {
                continue;
            }
            $output->_set($key, $value);
        }
        return $output;
    }
    /**
     * @inheritDoc
     */
    public function map (Closure $process): IImmutableCollection {
        $output = new static();
        foreach ($this->elements as $key => $value) {
            $output->_set($key, $process($key, $value));
        }
        return $output;
    }

    /**
     * @inheritDoc
     */
    public function keys (): IImmutableCollection {
        return new static(array_keys($this->elements));
    }
    /**
     * @inheritDoc
     */
    public function values (): IImmutableCollection {
        return new static(array_values($this->elements));
    }

    /**
     * @inheritDoc
     *
     * @throws JsonException
     */
    public function jsonSerialize () {
        return json_encode($this->elements, JSON_THROW_ON_ERROR);
    }
}

==> 2.1.0.zip: phpcollections/src/Collections/ImmutableCollection.php <==
<?php

namespace jrosset\Collections;

use ArrayIterator;
use Closure;
use InvalidArgumentException;
use JsonException;
use Traversable;

/**
 * An immutable collection
 *
 * @psalm-template     TKey of array-key
 * @template-covariant TValue
 * @template-implements IImmutableCollection<TKey, TValue>
 */
class ImmutableCollection implements IImmutableCollection {
    /**
     * @var array The internal array of elements
     * @psalm-var array<TKey, TValue>
     *
     * @internal
     */
    protected array $elements;

    /**
     * Initialise a new collection
     *
     * @param array|Traversable|null $other The initial values
     *
     * @psalm-param array<TKey, TValue>|Traversable<TKey, TValue>|null
     *
     * @throws InvalidArgumentException If the initial values aren't valid
     */
    public function __construct ($other = null) {
        $this->_initialize($other);
    }

    /**
     * Initialise the internals elements
     *
     * @param array|Traversable|null $other The initial values
     *
     * @psalm-param array<TKey, TValue>|Traversable<TKey, TValue>|null
     *
     * @throws InvalidArgumentException If the initial values aren't valid
     * @internal
     *
     */
    protected function _initialize ($other = null) {
        $this->elements = [];
        if ($other !== null) {
            if (!is_array($other) && !$other instanceof Traversable) {
                throw new InvalidArgumentException('The initial values must be an array or a ' . IArrayCast::class . ' instance');
            }

            foreach ($other as $key => $value) {
                $this->_set($key, $value);
            }
        }
    }
    /**
     * Internally set a value to a key
     *
     * @param array-key    $key   The key
     * @param mixed        $value The value
     *
     * @psalm-param TKey   $key
     * @psalm-param TValue $value
     *
     * @return $this
     * @internal
     *
     */
    protected function _set ($key, $value): self {
        $this->elements[$this->_normalizeKey($key)] = $value;
        return $this;
    }
    /**
     * Internally add a value
     *
     * @param mixed        $value The value
     *
     * @psalm-param TValue $value
     *
     * @return $this
     * @internal
     *
     */
    protected function _add ($value): self {
        $this->elements[] = $value;
        return $this;
    }
    /**
     * Normalize a key
     *
     * @param array-key  $key The key to normalize
     *
     * @psalm-param TKey $key
     *
     * @return array-key The normalized key
     * @psalm-return TKey
     */
    protected function _normalizeKey ($key) {
        return $key;
    }

    /**
     * @inheritDoc
     */
    public function getIterator () {
        return new ArrayIterator($this->elements);
    }

    /**
     * @inheritDoc
     */
    public function offsetExists ($offset): bool {
        return $this->exists($offset);
    }
    /**
     * @inheritDoc
     */
    public function offsetGet ($offset) {
        return $this->get($offset);
    }
    /**
     * @inheritDoc
     */
    public function offsetSet ($offset, $value): void {
        throw new ImmutableException();
    }
    /**
     * @inheritDoc
     */
    public function offsetUnset ($offset): void {
        throw new ImmutableException();
    }

    /**
     * @inheritDoc
     */
    public function serialize (): ?string {
        return serialize($this->elements);
    }
    /**
     * @inheritDoc
     */
    public function unserialize ($data) {
        $this->_initialize(unserialize($data));
    }

    /**
     * @inheritDoc
     */
    public function toArray (): array {
        return $this->elements;
    }

    /**
     * @inheritDoc
     */
    public function count (): int {
        return count($this->elements);
    }
    /**
     * @inheritDoc
     */
    public function empty (): bool {
        return $this->count() === 0;
    }

    /**
     * @inheritDoc
     */
    public function exists ($key): bool {
        return isset($this->elements[$this->_normalizeKey($key)]);
    }
    /**
     * @inheritDoc
     */
    public function contains ($value, bool $strict = false): bool {
        return in_array($value, $this->elements, $strict);
    }

    /**
     * @inheritDoc
     */
    public function get ($key) {
        return $this->elements[$this->_normalizeKey($key)] ?? null;
    }
    /**
     * @inheritDoc
     */
    public function key ($value, bool $strict = false) {
        foreach ($this->elements as $currentKey => $currentValue) {
            if (($strict && $value === $currentValue) || (!$strict && $value == $currentValue)) {
                return $currentKey;
            }
        }
        return null;
    }

    /**
     * @inheritDoc
     */
    public function slice (int $offset, ?int $length = null): self {
        $output = new static();

        $currentIndex = 0;
        foreach ($this->elements as $key => $value) {
            if ($currentIndex++ < $offset) {
                continue;
            }

            $output->_set($key, $value);
            if ($output->count() >= $length) {
                break;
            }
        }

        return $output;
    }

    /**
     * @inheritDoc
     */
    public function filter (Closure $filter): self {
        $output = new static();
        foreach ($this->elements as $key => $value) {
            if (!$filter($key, $value)) {
                continue;
            }
            $output->_set($key, $value);
        }
        return $output;
    }
    /**
     * @inheritDoc
     */
    public function withoutEmpties (): self {
        return $this->filter(function ($value) {
            return empty($value);
        });
    }
    /**
     * @inheritDoc
     */
    public function removeEmpties (): self {
        $this->_initialize($this->withoutEmpties());
        return $this;
    }
    /**
     * @inheritDoc
     */
    public function map (Closure $process): self {
        $output = new static();
        foreach ($this->elements as $key => $value) {
            $output->_set($key, $process($key, $value));
        }
        return $output;
    }

    /**
     * @inheritDoc
     */
    public function keys (): self {
        return new static(array_keys($this->elements));
    }
    /**
     * @inheritDoc
     */
    public function values (): self {
        return new static(array_values($this->elements));
    }

    /**
     * @inheritDoc
     *
     * @throws JsonException
     */
    public function jsonSerialize () {
        return json_encode($this->elements, JSON_THROW_ON_ERROR);
    }

    /**
     * @inheritDoc
     */
    public function join (string $separator): string {
        return implode($separator, $this->toArray());
    }
    /**
     * @inheritDoc
     */
    public static function split (string $value, string $separator): self {
        return new static (explode($separator, $value));
    }
}
